#include "alcstats.h"
#include "serv.h"
#include "parse.h"
#include "list.h"
#include "hash.h"
#include "balloc.h"
#include "servlist.h"
#include "userlist.h"
#include "domainlist.h"
#include "connlist.h"
#include "conf.h"
#ifdef USE_RRD
#include "rrdtool.h"
#endif

struct opt		opts;
struct pollfd		pollfd[1];
struct total_struct	totals;
pid_t			pid;
time_t			start;
int			stop;

void die(const char *pattern, ...)
{
	va_list va;

	va_start(va, pattern);
	vfprintf(stderr, pattern, va);
	va_end(va);
	fflush(stderr);
	exit(2);
}

void downstring(char *str)
{
	for (; *str!='\0'; ++str) {
		*str = tolower(*str);
	}
}

void sigterm_handler(int sig)
{
	stop = 1;
}

void setup_signal(void)
{
	struct sigaction act;

	act.sa_flags = SA_RESTART;
	act.sa_handler = SIG_IGN;
	sigemptyset(&act.sa_mask);
	sigaddset(&act.sa_mask, SIGHUP);
	sigaddset(&act.sa_mask, SIGPIPE);
	sigaction(SIGHUP, &act, 0);
	sigaction(SIGPIPE, &act, 0);
	act.sa_handler = sigterm_handler;
	sigaddset(&act.sa_mask, SIGTERM);
	sigaction(SIGTERM, &act, 0);
	sigaddset(&act.sa_mask, SIGINT);
	sigaction(SIGINT, &act, 0);
}

char *host_to_domain(const char *s, char *nw)
{
	char	*p, *q, xx[HOSTLEN+1];
	int	len;

	if ((q = strchr(s, '.')) == NULL)
		return NULL;

	if (strchr(q+1, '.') == NULL) {
		/* form yy.com -> don't truncate */
		mystrlcpy(nw, s, HOSTLEN);
		return nw;
	}

	len = strlen(s);
	if (len > HOSTLEN)
		len = HOSTLEN;
	memcpy(xx, s, len);
	xx[len] = '\0';
	if ((xx[len-1] >= '0') && (xx[len-1] <= '9'))
		return NULL;
	/* form yy.zz.etc.edu -> zz.etc.edu */
	p = rindex(xx, '.'); *p = '\0'; p += 1;
	q = rindex(xx, '.'); *q = '\0'; q += 1;
	snprintf(nw, HOSTLEN+1, "%s.%s", q, p);
	return(nw);
}

void *MyMalloc(size_t size)
{
	void *ptr;
	if ((ptr = calloc(1, size)) == NULL)
		die("Out of memory!\n");
	return ptr;
}

void MyFree(void *ptr)
{
	if (ptr != NULL) {
		free(ptr);
		ptr = NULL;
	}
}

char *mystrdup(char *src)
{
	char *ptr;
	if ((ptr = strdup(src)) == NULL)
		die("Out of memory!\n");
	return ptr;
}

char *mystrlcpy(char *dest, const char *src, size_t size)
{
	size_t ret, len;

	if (src == NULL) {
		dest = NULL;
		return NULL;
	}

	ret = strlen(src);
	len = (ret > size) ? size-1 : ret+1;

	memcpy(dest, src, len);
	dest[len-1] = '\0';
	return dest;
}

void clear_opts(void)
{
	optind = 0;
	opterr = 0;
	optopt = 0;
	optarg = NULL;
}

void parse_cmdline(int argc, char *argv[])
{
	int c;

	while ((c = getopt(argc, argv, "df?c:l:")) != EOF)
		switch(c) {
			case 'c':
				MyFree(opts.conffile);
				opts.conffile = mystrdup(optarg);
				break;
			case 'd':
				opts.debug=1;
				opts.fork=0;
				break;
			case 'f':
				opts.fork=0;
				break;
			case 'l':
				opts.logfile = mystrdup(optarg);
				break;
			case '?':
				fprintf(stderr, "Usage: %s [-d]|[-f] [-p port]\n", argv[0]);
				fprintf(stderr, "-d - Debugging (implies -f)\n");
				fprintf(stderr, "-f - Don't fork\n");
				fprintf(stderr, "-h <host> - Connect to <host>\n");
				fprintf(stderr, "-l <log> - Log to <log>\n");
				fprintf(stderr, "-p <port> - Connect to <port>\n");
				exit(1);
				break;
		}
	clear_opts();
}

int main(int argc, char *argv[])
{
	umask(0);
	close(STDIN_FILENO);
	stop = 0;
	memset(&opts, 0, sizeof(struct opt));
	memset(&conf, 0, sizeof(struct confitems));
	opts.fork = 1;
	conf.serverhsize = 1;
	conf.domainhsize = 1;
	conf.userhsize = 1;
	opts.conffile = mystrdup(CONFFILE);
#ifndef NOBALLOC
	initBlockHeap();
#endif
	list_init();
	init_conf_parse(conftab);
	do_conf(opts.conffile);
	parse_cmdline(argc, argv);
	snprintf(interface, USERLEN + HOSTLEN + 2, "%s@%s", conf.username, conf.servname);

	if (!opts.debug)
		close(STDOUT_FILENO);

	if (opts.fork) {
		pid = fork();
		if (pid < 0)
			die("Unable to fork!\n");
		else if (pid) {
			printf("Forked off child process: %d\n", (int)pid);
			return(0);
		}
		setsid();
		setpgid(0, 0);
	}

	setup_signal();
	init_tree_parse(msgtab);
	domain_init();
	server_init();
	user_init();
	init_totals();
	iovs = 0;
	read_data();
#ifdef USE_RRD
	makerrd(conf.network);
#endif
	start = time(NULL);
	nextc = start + 60; /* Force a check in 60 seconds */
	cur_conn = conns;
	connected = 0;
	ioloop(); 
	return(1);
}

